# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013, 2015, 2018 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.processor import complete as Mcomplete


class InfoBreak(Mbase_subcmd.DebuggerSubcommand):
    """**info breakpoints** [ *bp-number...* ]

Show the status of specified breakpoints (or all user-settable
breakpoints if no argument).

The "Disp" column contains one of "keep", or "del", to indicate the
disposition of the breakpoint after it gets hit.  "del" means that the
breakpoint will be deleted.  The ""Enb" column indicates if the
breakpoint is enabled. The "Where" column indicates the file/line
number of the breakpoint.

Also shown are the number of times the breakpoint has been hit,
when that count is at least one, and any conditions the breakpoint
has.

Example:
--------


    (trepan3k) info break
    Num Type          Disp Enb    Where
    1   breakpoint    del  n   at /tmp/fib.py:9
    2   breakpoint    keep y   at /tmp/fib.py:4
            breakpoint already hit 1 time
    3   breakpoint    keep y   at /tmp/fib.py:6
            stop only if x > 0

See also:
---------
`break`, `delete`, `enable`, `disable`, `condition`

    """

    min_abbrev = 1  # Min is info b
    need_stack = False
    short_help = "Status of user-settable breakpoints"

    complete = Mcomplete.complete_bpnumber

    def bpprint(self, bp):
        if bp.temporary:
            disp = 'del  '
        else:
            disp = 'keep '
            pass
        if bp.enabled:
            disp = disp + 'y  '
        else:
            disp = disp + 'n  '
            pass
        self.msg('%-4dbreakpoint    %s at %s:%d' %
                 (bp.number, disp, self.core.filename(bp.filename), bp.line))
        if bp.condition:
            self.msg('\tstop only if %s' % (bp.condition))
            pass
        if bp.ignore:
            self.msg('\tignore next %d hits' % (bp.ignore))
            pass
        if (bp.hits):
            if (bp.hits > 1): ss = 's'
            else: ss = ''
            self.msg('\tbreakpoint already hit %d time%s' %
                     (bp.hits, ss))
            pass
        return

    def run(self, args):
        bpmgr = self.core.bpmgr
        bpnums = bpmgr.bpnumbers()
        if len(bpnums) > 0:  # There's at least one
            if len(args) > 0:
                list_bpnums = list(set(bpnums) & set(args))
                if len(list_bpnums) == 0:
                    self.msg("No breakpoints in list given.")
                else:
                    for num_str in list_bpnums:
                        self.bpprint(bpmgr.get_breakpoint(num_str)[2])
                        pass
                    pass
                pass
            else:
                self.section("Num Type          Disp Enb    Where")
                for bp in bpmgr.bpbynumber:
                    if bp:
                        self.bpprint(bp)
                        pass
                    pass
                pass
            pass
        else:
            self.msg("No breakpoints.")
            pass
        return
    pass

if __name__ == '__main__':
    from trepan import debugger as Mdebugger
    from trepan.processor.command import info as Minfo
    d = Mdebugger.Trepan()
    i = Minfo.InfoCommand(d.core.processor)
    sub = InfoBreak(i)
    sub.run([])
    pass



#!/usr/bin/env python3
'Unit test for trepan.processor.cmdproc'
import unittest

from trepan.processor import cmdproc as Mcmdproc

from cmdhelper import dbg_setup


class TestProcesor(unittest.TestCase):
    
    def setUp(self):
        self.d, self.cp = dbg_setup()
    
    def test_populate_commands(self):
        """ Test that we are creating instances for all of classes of files
        in the command directory ."""
        for i in self.cp.cmd_instances:
            if hasattr(i, 'aliases'):
                self.assertEqual(tuple, type(i.aliases), 
                                 "not tuple %s." % repr(i.aliases))
                
                self.assertEqual([],
                                 [item for item in i.aliases 
                                  if str != type(item)],
                                 "elements of tuple should be strings %s" % 
                                 repr(i.aliases))
                pass
            pass
        return

    def test_get_commands_aliases(self):
        "Test that the command processor finds a command, alias, and method"
        self.assertTrue('quit' in list(self.cp.commands.keys()))
        self.assertEqual('quit', self.cp.aliases['q'])
        import inspect
        self.assertTrue(inspect.ismethod(self.cp.commands['quit'].run))
        return

    def test_resolve_name(self):
        "Test that the command processor finds a command, alias, and method"
        self.assertTrue(Mcmdproc.resolve_name(self.cp, 'quit'))
        self.assertTrue(Mcmdproc.resolve_name(self.cp, 'q'))
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2010, 2012-2013, 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.processor import cmdfns as Mcmdfns, cmdproc as Mcmdproc


class SetCmdDbgTrepan(Mbase_subcmd.DebuggerSetBoolSubcommand):
    """Set the ability to debug the debugger.

Setting this allows visibility and access to some of the debugger's
internals. Specifically variable "frame" contains the current frame and
variable "debugger" contains the top-level debugger object.
"""

    in_list    = True
    min_abbrev = len('dbg')    # Need at least "set dbg"

    def run(self, args):
        Mcmdfns.run_set_bool(self, args)
        if self.debugger.settings[self.name]:
            # Put a stack frame in the list of frames so we have
            # something to inspect.
            frame = inspect.currentframe()
            # Also give access to the top-level debugger
            self.proc.stack, self.proc.curindex = \
                Mcmdproc.get_stack(frame, None, self.proc)
            self.proc.curframe = self.proc.stack[self.proc.curindex][0]
            # Remove ignored debugger functions.
            self.save_ignore_filter = self.core.ignore_filter
            self.core.ignore_filter = None
        else:
            self.core.ignore_filter = self.save_ignore_filter
            pass

        return
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2012-2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''Things related to eval/exec'''

# extract the "expression" part of a line of source code.
#
import re


def extract_expression(text):
    if re.search('^\s*(?:if|elif)\s+', text):
        text = re.sub('^\s*(?:if|elif)\s+', '', text)
        text = re.sub(':(?:\s+.*$|$)', '', text)
    elif re.search('^\s*assert\s+.*', text):
        # EXPR in : assert EXPRESSION:
        text = re.sub('^\s*assert\s+', '', text)
    elif re.search('^\s*(?:while)\s+', text):
        text = re.sub('^\s*(?:while)\s+', '', text)
        text = re.sub(':(?:\s+.*$|$)', '', text)
    elif re.search('^\s*return\s+', text):
        # EXPRESION in: return EXPRESSION
        text = re.sub('^\s*return\s+', '', text)
    elif re.search('^\s*for\s+.+\s+in\s+.*:', text):
        # EXPRESION in: for VAR in EXPRESSION:
        text = re.sub('^\s*for\s+.+\s+in\s+', '', text)
        text = re.sub(':.*$', '', text)
    elif re.search('\s*[A-Za-z_][A-Za-z0-9_\[\]]*\s*=[^=>]', text):
        # RHS of an assignment statement.
        text = re.sub('^\s*[A-Za-z_][A-Za-z0-9_\[\]]*\s*=\s*', '', text)
        pass
    return text


# Demo it
if __name__=='__main__':
    for stmt in (
        'if condition(x):',
        'elif _is_magic(name):',
        'while expression:',
        'for i in range(3):',
        'abc = 123',
        'return return_value',
        'nothing_to_be.done'):
        print(extract_expression(stmt))
        pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2015, 2018, 2020 Rocky Bernstein
#

from pygments.styles import STYLE_MAP

# Our local modules
from trepan.processor.command.base_subcmd import DebuggerSubcommand
from trepan.lib.complete import complete_token


style_names = sorted(list(STYLE_MAP.keys()))


def complete(self, prefix):
    return complete_token(style_names)


class ShowStyle(DebuggerSubcommand):
    """**show style* *name*

Show the pygments style used in formatting 256-color terminal text.

See also:
---------

`set style`, `show highlight`
"""

    in_list = True
    min_abbrev = len("sty")
    short_help = "Set the pygments style"

    def run(self, args):
        if len(args) != 0:
            self.errmsg("Expecting no args")
            return

        style = self.debugger.settings[self.name]
        if style:
            self.msg("Pygments style is %s" % style)
        else:
            self.msg("Pygments style not set")
        return

    pass


if __name__ == "__main__":
    from trepan.processor.command.set_subcmd import __demo_helper__ as Mhelper

    sub = Mhelper.demo_run(ShowStyle, [])
    d = sub.proc.debugger
    sub.run(["show"])
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2014-2015, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""classes to support communication to and from the debugger.  This
communcation might be to/from another process or another computer.
And reading may be from a debugger command script.

For example, we'd like to support Sockets, and serial lines and file
reading, as well a readline-type input. Encryption and Authentication
methods might decorate some of the communication channels.

Some ideas originiated as part of Matt Fleming's 2006 Google Summer of
Code project.
"""

from abc import ABCMeta

NotImplementedMessage = "This method must be overriden in a subclass"


class DebuggerInputBase(metaclass=ABCMeta):
    """This is an abstract class that specifies debugger input."""

    def __init__(self, inp=None, opts=None):
        self.input = None
        return

    def close(self):
        if self.input:
            self.input.close()
            pass
        return

    def use_history(self):
        return False

    def open(self, inp, opts=None):
        """Use this to set where to read from. """
        raise NotImplementedError(NotImplementedMessage)

    def readline(self, use_raw=None):
        """Read a line of input. EOFError will be raised on EOF.

        Note that we don't support prompting first. Instead, arrange
        to call DebuggerOutput.write() first with the prompt. If
        `use_raw' is set raw_input() will be used in that is supported
        by the specific input input. If this option is left None as is
        normally expected the value from the class initialization is
        used.
        """
        raise NotImplementedError(NotImplementedMessage)

    pass


class DebuggerInOutBase(metaclass=ABCMeta):
    """ This is an abstract class that specifies debugger output. """

    def __init__(self, out=None, opts=None):
        self.output = None
        return

    def close(self):
        if self.output:
            self.output.close()
            pass
        return

    def flush(self):
        raise NotImplementedError(NotImplementedMessage)

    def write(self, output):
        """Use this to set where to write to. output can be a
        file object or a string. This code raises IOError on error.
        """
        raise NotImplementedError(NotImplementedMessage)

    def writeline(self, msg):
        """ used to write to a debugger that is connected to this
        server; `str' written will have a newline added to it
        """
        self.write("%s\n" % msg)
        return

    pass


class TrepanInOutBase(metaclass=ABCMeta):
    """ This is an abstract class that specifies debugger input output when
    handled by the same channel, e.g. a socket or tty.
    """

    def __init__(self, inout=None, opts=None):
        self.inout = None
        return

    def close(self):
        if self.inout:
            self.inout.close()
            pass
        return

    def flush(self):
        raise NotImplementedError(NotImplementedMessage)

    def open(self, inp, opts=None):
        """Use this to set where to read from. """
        raise NotImplementedError(NotImplementedMessage)

    def readline(self, use_raw=None):
        """Read a line of input. EOFError will be raised on EOF.

        Note that we don't support prompting first. Instead, arrange
        to call DebuggerOutput.write() first with the prompt. If
        `use_raw' is set raw_input() will be used in that is supported
        by the specific input input. If this option is left None as is
        normally expected the value from the class initialization is
        used.
        """
        raise NotImplementedError(NotImplementedMessage)

    def write(self, output):
        """Use this to set where to write to. output can be a
        file object or a string. This code raises IOError on error.
        """
        raise NotImplementedError(NotImplementedMessage)

    def writeline(self, msg):
        """ used to write to a debugger that is connected to this
        server; `str' written will have a newline added to it
        """
        self.write("%s\n" % msg)
        return

    pass


# Demo
if __name__ == "__main__":

    class MyInput(DebuggerInputBase):
        def open(self, inp, opts=None):
            print("open(%s) called" % inp)
            pass

        pass

    class MyOutput(DebuggerInOutBase):
        def writeline(self, s):
            print("writeline:", s)
            pass

        pass

    inp = MyInput()
    inp.open("foo")
    inp.close()
    out = MyOutput()
    out.writeline("foo")
    try:
        out.write("foo")
    except NotImplementedError:
        print("Ooops. Forgot to implement write()")
        pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013, 2015 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tracer

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd


class SetEvents(Mbase_subcmd.DebuggerSubcommand):

    """**set events** [*event* ...]

Sets the events that the debugger will stop on. Event names are:
`c_call`, `c_exception`, `c_return`, `call`, `exception`, `line`,
or `return`.

`all` can be used as an abbreviation for listing all event names.

Changing trace event filters works independently of turning on or off
tracing-event printing.

Examples:
---------

  set events line        # Set trace filter for line events only.
  set events call return # Trace calls and returns only
  set events all         # Set trace filter to all events.

See also:
---------

`set trace`, `show trace`, and `show events`. `help step` lists event names.
    """

    in_list    = True
    min_abbrev = len('ev')
    short_help = "Set execution-tracing event set"

    def run(self, args):
        valid_args = tracer.ALL_EVENT_NAMES + ('all',)
        eventset = []
        for arg in args:
            if arg not in valid_args:
                self.errmsg('set events: Invalid argument %s ignored.' % arg)
                continue
            if arg in tracer.ALL_EVENTS:
                eventset += [arg]
            elif 'all' == arg:
                eventset += tracer.ALL_EVENTS
            pass
        if [] != eventset:
            self.debugger.settings['printset'] = frozenset(eventset)
            pass
        return

    pass

if __name__ == '__main__':
    from trepan.processor.command import mock, set as Mset
    d, cp = mock.dbg_setup()
    s = Mset.SetCommand(cp)
    sub = SetEvents(s)
    sub.name = 'events'
    for args in (['line'], ['bogus'],
                ['call', 'return']):
        sub.run(args)
        print(d.settings['printset'])
        pass
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.cmdproc import print_location
from trepan.lib import bytecode as Mbytecode


class SkipCommand(DebuggerCommand):
    """**skip** [*count*]

Set the next line that will be executed. The line must be within the
stopped or bottom-most execution frame.

See also:
---------

`next`, `step`, `jump`, `continue`, `return` and
`finish` for other ways to progress execution.
"""

    aliases = ("sk",)
    execution_set = ["Running"]
    short_help = "Skip lines to be executed"

    DebuggerCommand.setup(locals(), category="running", max_args=1, need_stack=True)

    def run(self, args):
        if not self.core.is_running():
            return False

        if self.proc.curindex + 1 != len(self.proc.stack):
            self.errmsg("You can only skip within the bottom frame.")
            return False

        if self.proc.curframe.f_trace is None:
            self.errmsg("Sigh - operation can't be done here.")
            return False

        if len(args) == 1:
            count = 1
        else:
            msg = "skip: expecting a number, got %s." % args[1]
            count = self.proc.get_an_int(args[1], msg)
            pass
        co = self.proc.curframe.f_code
        offset = self.proc.curframe.f_lasti
        if count is None:
            return False
        lineno = Mbytecode.next_linestart(co, offset, count)

        if lineno < 0:
            self.errmsg("No next line found")
            return False

        try:
            # Set to change position, update our copy of the stack,
            # and display the new position
            self.proc.curframe.f_lineno = lineno
            self.proc.stack[self.proc.curindex] = (
                self.proc.stack[self.proc.curindex][0],
                lineno,
            )
            print_location(self.proc)
        except ValueError as e:
            self.errmsg("skip failed: %s" % e)
        return False

    pass


if __name__ == "__main__":
    from trepan.processor.command.mock import dbg_setup

    d, cp = dbg_setup()
    command = SkipCommand(cp)
    print("skip when not running: ", command.run(["skip", "1"]))
    command.core.execution_status = "Running"
    import inspect

    cp.curframe = inspect.currentframe()
    cp.curindex = 0
    from trepan.processor.cmdproc import get_stack

    cp.stack = get_stack(cp.curframe, None, None)
    command.run(["skip", "1"])
    cp.curindex = len(cp.stack) - 1
    command.run(["skip", "1"])
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'Unit test for trepan.inout.input'
import os, unittest

from trepan.inout import input as Minput

srcdir = os.path.abspath(os.path.dirname(__file__))


class TestDebuggerInput(unittest.TestCase):

    def test_DebuggerInput(self):
        cmdhelper_file = os.path.join(srcdir, 'cmdhelper.py')
        inp = Minput.DebuggerUserInput(cmdhelper_file)
        self.assertTrue(inp, 'Should have gotten a TrepanInput object back')
        line = inp.readline()
        self.assertEqual('# -*- coding: utf-8 -*-', line)
        inp.close()
        # Should be okay
        inp.close()
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand


class ExitCommand(DebuggerCommand):
    """**exit** [*exitcode*]

Hard exit of the debugged program.

The program being debugged is exited via *sys.exit()*. If a return code
is given, that is the return code passed to *sys.exit()*, the
return code that will be passed back to the OS.

See also:
---------

See `quit` and `kill`.
"""

    short_help = "Exit program via sys.exit()"

    DebuggerCommand.setup(locals(), category="support", max_args=1)

    def run(self, args):
        self.core.stop()
        self.core.execution_status = "Exit command"
        if len(args) <= 1:
            exit_code = 0
        else:
            exit_code = self.proc.get_int(args[1], default=0, cmdname="exit")
            if exit_code is None:
                return False
            pass
        # FIXME: allow setting a return code.
        import sys

        sys.exit(int(exit_code))
        # Not reached
        return True


# Demo it
if __name__ == "__main__":
    from trepan.processor.command import mock

    d, cp = mock.dbg_setup()
    command = ExitCommand(cp)
    command.run(["exit", "wrong", "number", "of", "args"])
    command.run(["exit", "foo"])
    command.run(["exit", "10"])



#!/usr/bin/env python3
import unittest
from fn_helper import compare_output, strarray_setup


class TestFinish(unittest.TestCase):
    def test_finish_same_level(self):
        print("test ", __file__, "finish_same_level skipped")
        return

        cmds = ['step 5', 'finish', 'continue']
        d = strarray_setup(cmds)
        d.core.start()

        def bar():
            x = 3  # NOQA
            return 1

        def foo():
            bar()
            x = 2  # NOQA
            return 5
        foo()
        d.core.stop()
        out = ['-- def foo():',
               '-- x = 3',
               '<- return 1',
               '<- return 5',  # FIXME this isn't right
               ]
        compare_output(self, out, d, cmds)

    # def test_finish_with_up(self):
    #     cmds = ['step 5', 'up', 'finish 2', 'continue']
    #     d = strarray_setup(cmds)
    #     d.core.start()
    #     def bar():
    #         x = 3
    #         return 1
    #     def foo():
    #         bar()
    #         x = 2
    #         return 5
    #     foo()
    #     d.core.stop()
    #     out = ['-- def foo():',
    #            '-- x = 3',
    #            '-- bar()',
    #            '<- return 5',
    #            ]
    #     compare_output(self, out, d, cmds)
    #     return

    pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013-2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Module for reading debugger scripts"""
import atexit

# Our local modules
from trepan import interface as Minterface, misc as Mmisc
from trepan.inout import scriptin as Mscriptin, output as Moutput

class ScriptInterface(Minterface.TrepanInterface):
    """Interface when reading debugger scripts"""

    DEFAULT_INIT_OPTS = {
        'abort_on_error' : True,
        'confirm_val'    : False,
        'verbose'        : False
        }

    def __init__(self, script_name, out=None, opts=None):
        get_option = lambda key: Mmisc.option_set(opts, key,
                                                  self.DEFAULT_INIT_OPTS)

        atexit.register(self.finalize)
        self.script_name     = script_name
        self.histfile        = None
        self.input_lineno    = 0
        self.input           = Mscriptin.ScriptInput(script_name)
        self.interactive     = False
        self.output          = out or Moutput.DebuggerUserOutput()

        self.abort_on_error  = get_option('abort_on_error')
        self.default_confirm = get_option('confirm_val')
        self.verbose         = get_option('verbose')

        return

    def close(self):
        """ Closes input. (We don't have an output.)"""
        self.input.close()
        return

    # Could look also look for interactive input and
    # use that. For now, though we'll simplify.
    def confirm(self, prompt, default):
        """ Called when a dangerous action is about to be done, to make
        sure it's okay. """
        return self.default_confirm

    def errmsg(self, msg, prefix="** "):
        """Common routine for reporting debugger error messages.
           """
        #  self.verbose shows lines so we don't have to duplicate info
        #  here. Perhaps there should be a 'terse' mode to never show
        #  position info.
        if not self.verbose:
            location = ("%s:%s: Error in source command file"
                        % (self.script_name, self.input_lineno))
            msg = "%s%s:\n%s%s" %(prefix, location, prefix, msg)
        else:
            msg = "%s%s" %(prefix, msg)
            pass
        self.msg(msg)
        if self.abort_on_error:
            raise EOFError
        return

    def finalize(self, last_wishes=None):
        # print exit annotation
        # save history
        self.close()
        return

    def read_command(self, prompt=''):
        '''Script interface to read a command. `prompt' is a parameter for
        compatibilty and is ignored.'''
        self.input_lineno += 1
        line = self.readline()
        if self.verbose:
            location = "%s line %s" % (self.script_name, self.input_lineno)
            self.msg('+ %s: %s' % (location, line))
            pass
        # Do something with history?
        return line

    # Could decide make this look for interactive input?
    def readline(self, prompt=''):
        '''Script interface to read a line. `prompt' is a parameter for
        compatibilty and is ignored.'''
        return self.input.readline()

# Demo
if __name__=='__main__':
    intf = ScriptInterface('script.py')
    line = intf.readline()
    print("Line read: %s" % line)
    pass



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2009-2010, 2013-2015, 2017-2018, 2020 Rocky Bernstein
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.cmdbreak import parse_break_cmd, set_break
from trepan.processor.complete import complete_break_linenumber


class BreakCommand(DebuggerCommand):
    """**break** [*location*] [if *condition*]]

Sets a breakpoint, i.e. stopping point just before the
execution of the instruction specified by *location*.

Without arguments or an empty *location*, the breakpoint is set at the
current stopped location.

See `help syntax location` for detailed information on a location.

If the word `if` is given after *location*, subsequent arguments given
Without arguments or an empty *location*, the breakpoint is set
the current stopped location.

Normally we only allow stopping at lines that we think are
stoppable. If the command has a `!` suffix, force the breakpoint anyway.

Examples:
---------

   break                # Break where we are current stopped at
   break if i < j       # Break at current line if i < j
   break 10             # Break on line 10 of the file we are
                        # currently stopped at
   break! 10            # Break where we are current stopped at, even if
                        # we don't think line 10 is stoppable
   break os.path.join() # Break in function os.path.join
   break x[i].fn()      # break in function specified by x[i].fn
   break x[i].fn() if x # break in function specified by x[i].fn
                        # if x is set
   break os.path:45     # Break on line 45 file holding module os.path
   break myfile.py:2    # Break on line 2 of myfile.py
   break myfile.py:2 if i < j # Same as above but only if i < j
   break "foo's.py":1"  # One way to specify path with a quote
   break 'c:\\foo.bat':1      # One way to specify a Windows file name,
   break '/My Docs/foo.py':1  # One way to specify path with blanks in it

See also:
---------

`info break`, `tbreak`, `condition` and `help syntax location`."""

    aliases = ("b", "break!", "b!")
    short_help = "Set breakpoint at specified line or function"

    DebuggerCommand.setup(locals(), category="breakpoints", need_stack=True)

    complete = complete_break_linenumber

    def run(self, args):
        force = True if args[0][-1] == "!" else False

        (func, filename, lineno, condition, offset) = parse_break_cmd(self.proc, args)
        if not (func == None and filename == None):
            set_break(
                self, func, filename, lineno, condition, False, args, force=force,
                offset=offset
            )
        return


if __name__ == "__main__":

    def doit(cmd, a):
        cmd.current_command = " ".join(a)
        print(parse_break_cmd(cmd.proc, a))

    import sys
    from trepan.debugger import Trepan

    d = Trepan()
    command = BreakCommand(d.core.processor)
    command.proc.frame = sys._getframe()
    command.proc.setup()

    doit(command, [" "])
    doit(command, ["10"])
    doit(command, [__file__ + ":10"])

    def foo():
        return "bar"

    doit(command, ["foo"])
    doit(command, ["os.path"])
    doit(command, ["os.path", "5+1"])
    doit(command, ["os.path.join"])
    doit(command, ["if", "True"])
    doit(command, ["foo", "if", "True"])
    doit(command, ["os.path:10", "if", "True"])
    command.run(["break"])
    command.run(["break", "command.run"])
    command.run(["break", "10"])
    command.run(["break", __file__ + ":10"])
    command.run(["break", "foo"])
    pass



#!/usr/bin/env python
import unittest
from fn_helper import compare_output, strarray_setup


class TestNext(unittest.TestCase):
    print("test ", __file__, "skipped")

    @unittest.skip("FIXME: figure out why this doesn't work")
    def test_next_same_level(self):

        # See that we can next with parameter which is the same as 'next 1'
        cmds = ['next', 'continue']
        d = strarray_setup(cmds)
        d.core.start()
        x = 5
        y = 6
        d.core.stop()
        out = ['-- x = 5',
               '-- y = 6']
        compare_output(self, out, d, cmds)

        # See that we can next with a computed count value
        cmds = ['next 5-3', 'continue']
        d = strarray_setup(cmds)
        d.core.start()
        x = 5  # NOQA
        y = 6  # NOQA
        z = 7  # NOQA
        d.core.stop(options={'remove': True})
        out = ['-- x = 5  # NOQA',
               '-- z = 7  # NOQA']
        compare_output(self, out, d, cmds)
        return

    @unittest.skip("FIXME: figure out why this doesn't work")
    def test_next_between_fn(self):
        # Next over a function
        def fact(x):
            if x <= 1: return 1
            return fact(x-1)
        cmds = ['next', 'continue']
        d = strarray_setup(cmds)
        d.core.start()
        x = fact(4)  # NOQA
        y = 5  # NOQA
        d.core.stop(options={'remove': True})
        out = ['-- x = fact(4)  # NOQA',
               '-- y = 5  # NOQA']
        compare_output(self, out, d, cmds)
        return

    def test_next_in_exception(self):
        return

        def boom(x):
            y = 0/x  # NOQA
            return

        def buggy_fact(x):
            if x <= 1: return boom(0)
            return buggy_fact(x-1)
        cmds = ['next', 'continue']
        d = strarray_setup(cmds)
        try:
            d.core.start()
            x = buggy_fact(4)  # NOQA
            y = 5  # NOQA
            self.assertTrue(False, 'should have raised an exception')
        except ZeroDivisionError:
            self.assertTrue(True, 'Got the exception')
        finally:
            d.core.stop(options={'remove': True})
            pass

        out = ['-- x = buggy_fact(4)  # NOQA',
               '!! x = buggy_fact(4)  # NOQA']
        compare_output(self, out, d, cmds)
        return

    pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Bytecode instruction routines"""

import xdis, re
from opcode import opname, HAVE_ARGUMENT


def op_at_code_loc(code, loc):
    try:
        op = code[loc]
    except IndexError:
        return "got IndexError"
    return opname[op]


def op_at_frame(frame, loc=None):
    code = frame.f_code.co_code
    if loc is None:
        loc = frame.f_lasti
    return op_at_code_loc(code, loc)


def next_opcode(code, offset):
    """Return the next opcode and offset as a tuple. Tuple (-100,
    -1000) is returned when reaching the end."""
    n = len(code)
    while offset < n:
        op = code[offset]
        offset += 1
        if op >= HAVE_ARGUMENT:
            offset += 2
            pass
        yield op, offset
        pass
    yield -100, -1000
    pass


def next_linestart(co, offset, count=1):
    linestarts = dict(xdis.findlinestarts(co))
    code = co.co_code
    # n = len(code)
    # contains_cond_jump = False
    for op, offset in next_opcode(code, offset):
        if offset in linestarts:
            count -= 1
            if 0 == count:
                return linestarts[offset]
            pass
        pass
    return -1000


def stmt_contains_opcode(co, lineno, query_opcode):
    linestarts = dict(xdis.findlinestarts(co))
    code = co.co_code
    found_start = False
    for offset, start_line in list(linestarts.items()):
        if start_line == lineno:
            found_start = True
            break
        pass
    if not found_start:
        return False
    for op, offset in next_opcode(code, offset):
        if -1000 == offset or linestarts.get(offset):
            return False
        opcode = opname[op]
        # debug: print opcode
        if query_opcode == opcode:
            return True
        pass
    return False


_re_def_str = r"^\s*def\s"
_re_def = re.compile(_re_def_str)


def is_def_stmt(line, frame):
    """Return True if we are looking at a def statement"""
    # Should really also check that operand of 'LOAD_CONST' is a code object
    return (
        line
        and _re_def.match(line)
        and op_at_frame(frame) == "LOAD_CONST"
        and stmt_contains_opcode(frame.f_code, frame.f_lineno, "MAKE_FUNCTION")
    )


_re_class = re.compile(r"^\s*class\s")


def is_class_def(line, frame):
    """Return True if we are looking at a class definition statement"""
    return (
        line
        and _re_class.match(line)
        and stmt_contains_opcode(frame.f_code, frame.f_lineno, "BUILD_CLASS")
    )


# Demo stuff above
if __name__ == "__main__":
    import inspect

    def sqr(x):
        return x * x

    frame = inspect.currentframe()
    co = frame.f_code
    lineno = frame.f_lineno
    print(
        "contains MAKE_FUNCTION %s"
        % stmt_contains_opcode(co, lineno - 4, "MAKE_FUNCTION")
    )
    print(
        "contains MAKE_FUNCTION %s" % stmt_contains_opcode(co, lineno, "MAKE_FUNCTION")
    )

    print("op at frame: %s" % op_at_frame(frame))
    print("op at frame, position 2: %s" % op_at_frame(frame, 2))
    print("def statement: x=5?: %s" % is_def_stmt("x=5", frame))
    # Not a "def" statement because frame is wrong spot
    print(is_def_stmt("def foo():", frame))

    class Foo:
        pass

    lineno = frame.f_lineno
    print(
        "contains BUILD_CLASS %s" % stmt_contains_opcode(co, lineno - 2, "BUILD_CLASS")
    )
    print("contains BUILD_CLASS %s" % stmt_contains_opcode(co, lineno, "BUILD_CLASS"))
    pass



#!/usr/bin/env python3
'Unit test for pydbgr.processor.cmdproc'
import sys, unittest
import os.path as osp

from trepan.processor import cmdproc as Mcmdproc
from trepan.processor.command import mock as Mmock
from trepan.processor.cmdbreak import parse_break_cmd

def canonic_tuple(t):
    fname = t[1]
    if fname:
        if fname.endswith('.pyc'):
            fname = fname[:-1]
        got = list(t)
        got[1] = osp.basename(fname)
        t = tuple(got)
    return t


class TestCmdParse(unittest.TestCase):

    def setUp(self):
        self.errors             = []
        self.msgs               = []
        self.d                  = Mmock.MockDebugger()
        self.cp                 = Mcmdproc.CommandProcessor(self.d.core)
        self.cp.intf[-1].msg    = self.msg
        self.cp.intf[-1].errmsg = self.errmsg
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msg.append(msg)
        return

    def test_basic(self):

        self.cp.frame = sys._getframe()
        self.cp.setup()
        for expect, cmd in (
                ( (None, None, None),
                  "break '''c:\\tmp\\foo.bat''':1" ),
                ( (None, None, None),
                  'break """/Users/My Documents/foo.py""":2' ),
                ( (None, osp.basename(__file__), 10),
                  "break 10" ),
                ( (None, None, None),
                   "break cmdproc.py:5" ) ,
                ( (None, None, None),
                   "break set_break()" ),
                ( (None, osp.basename(__file__), 4, 'i==5'),
                   "break 4 if i==5" ),
                ( (None, None, None),
                  "break cmdproc.setup()" ),
                ):
            args = cmd.split(' ')
            self.cp.current_command = cmd
            got = canonic_tuple(parse_break_cmd(self.cp, args))
            self.assertEqual(expect, tuple(got[:len(expect)]), cmd)
            # print(got)


        self.cp.frame = sys._getframe()
        self.cp.setup()

        # WARNING: magic number after f_lineno is fragile on the number of tests!
        # FIXME: can reduce by using .format() before test?
        break_lineno = self.cp.frame.f_lineno + 9
        for expect, cmd in (
                ( (None, osp.basename(__file__), break_lineno, None),
                    "break" ),
                ( (None, osp.basename(__file__), break_lineno, 'True'),
                    "break if True" ),
                ):
            args = cmd.split(' ')
            self.cp.current_command = cmd
            got = canonic_tuple(parse_break_cmd(self.cp, args))
            self.assertEqual(expect, got[:len(expect)])
            print(parse_break_cmd(self.cp, args))

        print(break_lineno)
        pass
        return


if __name__ == '__main__':
    unittest.main()
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.exception import DebuggerRestart


class RunCommand(DebuggerCommand):
    """**run**

Soft restart debugger and program via a *DebuggerRestart*
exception.

See also:
---------

`restart` for another way to restart the debugged program.

See `quit`, `exit` or `kill` for termination commands.
"""

    aliases = ("R",)
    short_help = "(Soft) restart program via a DebuggerRestart exception"

    DebuggerCommand.setup(locals(), category="support", max_args=0)

    def run(self, args):
        confirmed = self.confirm("Soft restart", False)
        if confirmed:
            self.core.step_ignore = 0
            self.core.step_events = None
            raise DebuggerRestart(self.core.debugger.restart_argv())
        pass

    pass


if __name__ == "__main__":
    from trepan.processor.command.mock import dbg_setup

    d, cp = dbg_setup()
    command = RunCommand(cp)
    try:
        command.run([])
    except DebuggerRestart:
        import sys

        print("Got restart exception: parms %s" % sys.exc_info()[1].sys_argv)
        pass
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2013, 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, types

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd


class MacroCommand(Mbase_cmd.DebuggerCommand):
    """**macro** *macro-name* *lambda-object*

Define *macro-name* as a debugger macro. Debugger macros get a list of
arguments which you supply without parenthesis or commas. See below
for an example.

The macro (really a Python lambda) should return either a String or an
List of Strings. The string in both cases is a debugger command.  Each
string gets tokenized by a simple split() .  Note that macro
processing is done right after splitting on `;;`. As a result, if the
macro returns a string containing `;;` this will not be interpreted as
separating debugger commands.

If a list of strings is returned, then the first string is
shifted from the list and executed. The remaining strings are pushed
onto the command queue. In contrast to the first string, subsequent
strings can contain other macros. `;;` in those strings will be
split into separate commands.

Here is an trivial example. The below creates a macro called `l=` which is
the same thing as `list`:

    macro l= lambda: 'list .'

A simple text to text substitution of one command was all that was
needed here. But usually you will want to run several commands. So those
have to be wrapped up into a list.

The below creates a macro called `fin+` which issues two commands
`finish` followed by `step`:

    macro fin+ lambda: ['finish','step']

If you wanted to parameterize the argument of the `finish` command
you could do that this way:

    macro fin+ lambda levels: ['finish %s' % levels ,'step']

Invoking with:

     fin+ 3

would expand to: `['finish 3', 'step']`

If you were to add another parameter for `step`, the note that the
invocation might be:

     fin+ 3 2

rather than `fin+(3,2)` or `fin+ 3, 2`.

See also:
---------

 `alias` and `info macro`.
  """

    category   = 'support'
    min_args   = 2  # Need at least this many: macro_name
    max_args   = None
    name       = os.path.basename(__file__).split('.')[0]
    need_stack  = False
    short_help  = 'Define a macro'

    def run(self, args):

        cmd_name = args[1]
        cmd_argstr = self.proc.cmd_argstr[len(cmd_name):].lstrip()
        proc_obj = None
        try:
            proc_obj = eval(cmd_argstr)
        except (SyntaxError, NameError, ValueError):
            self.errmsg("Expecting a Python lambda expression; got %s" % cmd_argstr)
            pass
        if proc_obj:
            if isinstance(proc_obj, types.FunctionType):
                self.proc.macros[cmd_name] = [proc_obj, cmd_argstr]
                self.msg("Macro \"%s\" defined." % cmd_name)
            else:
                self.errmsg("Expecting a Python lambda expression; got: %s" %
                            cmd_argstr)
                pass
            pass
        return
    pass

# Demo it
if __name__ == '__main__':
    from trepan.processor.command import mock as Mmock
    dbgr, cmd = Mmock.dbg_setup()
    command = MacroCommand(cmd)
    for cmdline in ["macro foo lambda a,y: x+y",
                    "macro bad2 1+2"]:
        args = cmdline.split()
        cmd_argstr = cmdline[len(args[0]):].lstrip()
        cmd.cmd_argstr = cmd_argstr
        command.run(args)
        pass
    print(cmd.macros)
    pass



#  Copyright (C) 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.lib.complete import complete_token


class UnaliasCommand(DebuggerCommand):
    """**unalias** *alias-name*

Remove alias *alias-name*

See also:
---------

'alias'
"""

    min_args      = 1
    short_help    = 'Remove an alias'

    DebuggerCommand.setup(
        locals(), category="support", max_args=1
    )
    def complete(self, prefix):
        return complete_token(self.proc.aliases.keys(), prefix)

    # Run command.
    def run(self, args):
        for arg in args[1:]:
            if arg in self.proc.aliases:
                del(self.proc.aliases[arg])
                self.msg("Alias for %s removed." % arg)
            else:
                self.msg("No alias found for %s" % arg)
                pass
            pass
        return
    pass

if __name__ == '__main__':
    # Demo it.
    from trepan import debugger
    d            = debugger.Trepan()
    cp           = d.core.processor
    command      = UnaliasCommand(cp)
    command.run(['unalias', 's'])
    command.run(['unalias', 's'])
    command.run(['unalias', 'foo', 'n'])
    print(command.complete(''))
    pass



#!/usr/bin/env python3
'Unit test for trepan.lib.eval'
import unittest

from trepan.lib import eval as Meval


class TestExtractExpression(unittest.TestCase):
    def test_extract_expression(self):
        for fragment, expect in (
            ('if condition(x):',     'condition(x)'),
            ('elif is_magic(name):', 'is_magic(name)'),
            ('while expression:',    'expression'),
            ('for i in range(3):',   'range(3)'),
            ('abc = 123',            '123'),
            ('assert True',          'True'),
            ('return return_value',  'return_value'),
            ('nothing_to_be.done',   'nothing_to_be.done'), ):
            self.assertEqual(expect , Meval.extract_expression(fragment))
            pass
        pass
    pass

if __name__ == '__main__':
    unittest.main()



class Token:
    """
    Class representing a token.
    kind: the kind of token, e.g. filename, number, other
    value: specific instance value, e.g. "/tmp/foo.c", or 5
    offset: byte offset from start of parse string
    """
    def __init__(self, kind, value=None, offset=None):
        self.offset = offset
        self.kind = kind
        self.value = value

    def __eq__(self, o):
        """ '==', but it's okay if offset is different"""
        if isinstance(o, Token):
            # Both are tokens: compare kind and value
            # It's okay if offsets are different
            return (self.kind == o.kind)
        else:
            return self.kind == o

    def __repr__(self):
        return str(self.kind)

    def __repr1__(self, indent, sib_num=''):
        return self.format(line_prefix=indent, sib_num=sib_num)

    def __str__(self):
        return self.format(line_prefix='')

    def format(self, line_prefix='', sib_num=None):
        if sib_num:
            sib_num = "%d." % sib_num
        else:
            sib_num = ''
        prefix = ('%s%s' % (line_prefix, sib_num))
        offset_opname = '%5s %-10s' % (self.offset, self.kind)
        if not self.value:
            return "%s%s" % (prefix, offset_opname)
        return "%s%s %s" % (prefix, offset_opname,  self.value)

    def __hash__(self):
        return hash(self.kind)

    def __getitem__(self, i):
        raise IndexError



#!/usr/bin/env python3
'Unit test for trepan.interfaces.user'
import unittest

from trepan.interfaces import user as Muser

from cmdhelper import dbg_setup

class TestInterfaceUser(unittest.TestCase):
    """Tests UserInterface class"""

    def readline(self, answer):
        return answer

    def test_confirm(self):
        """Test interface.user.UserInterface.confirm()"""
        d, cp = dbg_setup()
        u = Muser.UserInterface()
        for s in ['y', 'Y', 'Yes', '  YES  ']:
            u.input.readline = lambda prompt=None: self.readline(s)
            self.assertTrue(u.confirm('Testing', True))
            pass
        for s in ['n', 'N', 'No', '  NO  ']:
            u.input.readline = lambda prompt=None: self.readline(s)
            self.assertFalse(u.confirm('Testing', True))
            pass
        # FIXME: Add checking default values. Checking looping
        # values
        return
    # FIXME: more thorough testing of other routines in user.

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2017-2018, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from sys import version_info
from trepan.processor.command.base_cmd import DebuggerCommand
from uncompyle6.semantics.fragments import deparse_code
from xdis import IS_PYPY
from uncompyle6.semantics.fragments import deparsed_find


class DEvalCommand(DebuggerCommand):
    """**deval**
    **deval?**

Run a the current deparsed expression in the context of the current
frame. Normally we are stopped before an expression so the thing that
corresponds to the `eval` command is running the parent
construct. `deval?` will run just the command associated with the next
piece of code to be run.

Examples:
---------

    deval   # Run *parent* of current deparsed code
    deval?  # Run current deparsed code

See also:
---------

`eval`

    """

    aliases = ("deval?",)
    short_help = "Print value of deparsed expression"

    DebuggerCommand.setup(locals(), category="data", need_stack=True, max_args=0)

    def run(self, args):
        co = self.proc.curframe.f_code
        name = co.co_name
        last_i = self.proc.curframe.f_lasti
        if last_i == -1:
            last_i = 0

        sys_version = version_info[0] + (version_info[1] / 10.0)
        try:
            deparsed = deparse_code(sys_version, co, is_pypy=IS_PYPY)
            nodeInfo = deparsed_find((name, last_i), deparsed, co)
            if not nodeInfo:
                self.errmsg("Can't find exact offset %d" % last_i)
                return
        except:
            self.errmsg("error in deparsing code")
            return
        if "?" == args[0][-1]:
            extractInfo = deparsed.extract_node_info(nodeInfo.node)
        else:
            extractInfo, _ = deparsed.extract_parent_info(nodeInfo.node)
        text = extractInfo.selectedText
        text = text.strip()
        self.msg("Evaluating: %s" % text)
        try:
            self.proc.exec_line(text)
        except:
            pass


if __name__ == "__main__":
    import inspect
    from trepan import debugger

    d = debugger.Trepan()
    cp = d.core.processor
    cp.curframe = inspect.currentframe()
    command = DEvalCommand(cp)
    me = 10

    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2016 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd


class SetBasename(Mbase_subcmd.DebuggerSetBoolSubcommand):
    """**set basename** [ **on** | **off** ]

Set basename (short filenames) in debugger output.

Setting this causes the debugger output to give just the basename for
filenames. This is useful in debugger testing or possibly showing
examples where you don't want to hide specific filesystem and
installation information."""

    in_list    = True
    min_abbrev = len('ba')
    pass

if __name__ == '__main__':
    from trepan.processor.command.set_subcmd import __demo_helper__ as Mhelper
    Mhelper.demo_run(SetBasename)
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from trepan.processor.frame import adjust_relative
from trepan.processor.command.up import UpCommand


class DownCommand(UpCommand):

    signum = 1  # This is what distinguishes us from "up".
    short_help = "Move stack frame to a more recent selected frame"

    UpCommand.setup(locals(), category="stack", need_stack=True, max_args=1)

    def run(self, args):
        """**down** [*count*]

Move the current frame down in the stack trace (to a newer frame). 0
is the most recent frame. If no count is given, move down 1.

See also:
---------

`up` and `frame`."""

        adjust_relative(self.proc, self.name, args, self.signum)
        return False


if __name__ == "__main__":
    import inspect
    from trepan.processor.cmdproc import get_stack
    from trepan.debugger import Trepan

    d = Trepan()
    cp = d.core.processor
    command = DownCommand(cp)
    command.run(["down"])

    def nest_me(cp, command, i):
        if i > 1:
            cp.curframe = inspect.currentframe()
            cp.stack, cp.curindex = get_stack(cp.curframe, None, None, cp)
            command.run(["down"])
            print("-" * 10)
            command.run(["down", "1"])
            print("-" * 10)
            command.run(["down", "-1"])
            print("-" * 10)
        else:
            nest_me(cp, command, i + 1)
        return

    cp.forget()
    nest_me(cp, command, 1)
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013, 2015, 2018 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.lib import pp as Mpp


class InfoGlobals(Mbase_subcmd.DebuggerSubcommand):
    """**info globals** [*var1 ...*]

**info globals** *

With no arguments, show all of the global variables of the current stack
frame. If a list of names is provide limit display to just those
variables.

If `*` is given, just show the variable names, not the values.

See also:
---------
`info locals`, `info args`, `info frame`"""
    min_abbrev = 2
    need_stack = True
    short_help = '''Show the debugged programs's global variables'''

    def run(self, args):
        if not self.proc.curframe:
            self.errmsg("No frame selected.")
            return False
        names = list(self.proc.curframe.f_globals.keys())
        if len(args) > 0 and args[0] == '*' :
            self.section("globals")
            self.msg(self.columnize_commands(names))
        elif len(args) == 0:
            names.sort()
            for name in sorted(names):
                val = self.proc.getval(name)
                Mpp.pp(val, self.settings['width'], self.msg_nocr, self.msg,
                       prefix='%s =' % name)
                pass
        else:
            for name in args:
                if name in names:
                    val = self.proc.getval(name)
                    Mpp.pp(val, self.settings['width'], self.msg_nocr,
                           self.msg, prefix='%s =' % name)
                    pass
                else:
                    self.errmsg("%s is not a global variable" % name)
                    pass
                pass
        return False
    pass

if __name__ == '__main__':
    from trepan.processor.command import mock, info as Minfo
    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoGlobals(i)
    import inspect
    cp.curframe = inspect.currentframe()
    sub.run([])



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013, 2015, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import inspect, re
import os.path as osp

# Our local modules
from trepan.processor.command.base_subcmd import DebuggerSubcommand
from trepan.clifns import search_file
from trepan.misc import wrapped_lines
from trepan.processor.cmdbreak import parse_break_cmd
from pyficache import code_line_info


def find_function(funcname, filename):
    cre = re.compile(r"def\s+%s\s*[(]" % re.escape(funcname))
    try:
        fp = open(filename)
    except IOError:
        return None
    # consumer of this info expects the first line to be 1
    lineno = 1
    answer = None
    while True:
        line = fp.readline()
        if line == "":
            break
        if cre.match(line):
            answer = funcname, filename, lineno
            break
        lineno = lineno + 1
        pass
    fp.close()
    return answer


class InfoLine(DebuggerSubcommand):
    """**info line* [*location*]

Show line information for location *location*.

If no location is given, use the the current stopped line.

Examples
--------

    (trepan3k) info line
    Line 3 of "/tmp/python3-trepan/test/example/multi-line.py"
        starts at offset 0 of <module> and contains 2 instructions
    There are multiple starting offsets this line. Other starting offsets: 4 of <module>

    (trepan3k) info line 5
    Line 5 of "/tmp/python3-trepan/test/example/multi-line.py"
        starts at offset 16 of <module> and contains 7 instructions

See also:
---------
`info program`, `info frame`"""

    min_abbrev = 2
    max_args = 4
    need_stack = False
    short_help = "Show line information"

    def lineinfo(self, arg):
        (func, filename, lineno, condition, offset) = parse_break_cmd(
            self.proc, ["info args"]
        )
        if filename != None and lineno != None:
            return lineno, filename
        else:
            return None, None

    def run(self, args):
        """Current line number in source file"""
        if not self.proc.curframe:
            self.errmsg("No line number information available.")
            return

        # info line <loc>
        if len(args) == 0:
            # No line number. Use current frame line number
            line_number = inspect.getlineno(self.proc.curframe)
            filename = self.core.canonic_filename(self.proc.curframe)
        elif len(args) == 1:
            # lineinfo returns (item, file, lineno) or (None,)
            line_number, filename = self.lineinfo(args[2:])
            if not filename:
                self.errmsg("Can't parse '%s'" % args[2])
                pass
            filename = self.core.canonic(filename)
        else:
            self.errmsg("Wrong number of arguments.")
            return
        if not osp.isfile(filename):
            filename = search_file(filename, self.core.search_path, self.main_dirname)
            pass

        line_info = code_line_info(filename, line_number)
        msg1 = 'Line %d of "%s"' % (line_number, self.core.filename(filename),)
        if line_info:
            msg2 = "starts at offset %d of %s and contains %d instructions" % (
                line_info[0].offsets[0],
                line_info[0].name,
                len(line_info[0].offsets),
            )
            self.msg(wrapped_lines(msg1, msg2, self.settings["width"]))
        else:
            self.errmsg(
                "No line information for line %d of %s"
                % (line_number, self.core.filename(filename))
            )
        if line_info and len(line_info) > 1:
            self.msg(wrapped_lines(
                "There are multiple starting offsets this line.",
                "Other starting offsets: %s"
                % ", ".join(
                    ["%s of %s" % (li.offsets[0], li.name) for li in line_info[1:]]),
                self.settings["width"]
                )
            )
        return False

    pass


if __name__ == "__main__":
    from trepan.processor.command import mock
    from trepan.processor.command.info import InfoCommand
    from trepan.debugger import Trepan

    d = Trepan()
    d, cp = mock.dbg_setup(d)
    i = InfoCommand(cp)
    sub = InfoLine(i)
    cp.curframe = inspect.currentframe()
    for width in (80, 200):
        sub.settings["width"] = width
        sub.run([])
        line = inspect.getlineno(cp.curframe)
        x = 1
        cp.current_command = "info args %d" % line
        sub.run([line])
        pass
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import atexit, os

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.misc import wrapped_lines


class RestartCommand(DebuggerCommand):
    """**restart**

Restart debugger and program via an *exec()* call. All state is lost,
and new copy of the debugger is used.

See also:
---------

`run` for another way to restart the debugged program.

See `quit`, `exit` or `kill` for termination commands."""

    short_help = "(Hard) restart of program via execv()"

    DebuggerCommand.setup(locals(), category="support", max_args=0)

    def run(self, args):
        sys_argv = self.debugger.restart_argv()
        if sys_argv and len(sys_argv) > 0:
            confirmed = self.confirm("Restart (execv)", False)
            if confirmed:
                self.msg(
                    wrapped_lines(
                        "Re exec'ing:", repr(sys_argv), self.settings["width"]
                    )
                )
                # Run atexit finalize routines. This seems to be Kosher:
                # http://mail.python.org/pipermail/python-dev/2009-February/085791.html # NOQA
                try:
                    atexit._run_exitfuncs()
                except:
                    pass
                os.execvp(sys_argv[0], sys_argv)
                pass
            pass
        else:
            self.errmsg("No executable file and command options recorded.")
            pass
        return

    pass


if __name__ == "__main__":
    from trepan.processor.command.mock import dbg_setup

    d, cp = dbg_setup()
    command = RestartCommand(cp)
    command.run([])
    import sys

    if len(sys.argv) > 1:
        # Strip of arguments so we don't loop in exec.
        d.orig_sys_argv = ["python", sys.argv[0]]
        command.run([])
        pass
    pass



#!/usr/bin/env python3
'Unit test for pydbgr.processor.cmdproc'
import sys, unittest

from trepan.processor import cmdproc as Mcmdproc
from trepan.processor.command import mock as Mmock


class TestCmdProc(unittest.TestCase):

    def setUp(self):
        self.errors             = []
        self.msgs               = []
        self.d                  = Mmock.MockDebugger()
        self.cp                 = Mcmdproc.CommandProcessor(self.d.core)
        self.cp.intf[-1].msg    = self.msg
        self.cp.intf[-1].errmsg = self.errmsg
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msg.append(msg)
        return

    def test_basic(self):

        # We assume there's at least one command
        self.assertTrue(len(self.cp.commands) > 0)
        self.assertTrue(len(self.cp.aliases) > 0)

        # In fact we assume there is a 'quit' command...
        self.assertEqual('quit', Mcmdproc.resolve_name(self.cp, 'quit'))
        #   with alias 'q'
        self.assertEqual('quit', Mcmdproc.resolve_name(self.cp, 'q'))
#         processor.cmdproc.print_source_line(self.msg, 100,
#                                             'source_line_test.py')

        self.cp.frame = sys._getframe()
        self.cp.setup()
        self.assertEqual(3, self.cp.eval('1+2'))
        return

    def test_class_vars(self):
        '''See that each command has required attributes defined.  Possibly in
        a strongly-typed language we would not need to do much of this.'''

        for cmd in list(self.cp.commands.values()):

            name = cmd.__class__
            for attr in ['aliases', 'min_args', 'max_args', 'name',
                         'need_stack']:
                self.assertTrue(hasattr(cmd, attr),
                                '%s command should have a %s attribute' %
                                (name, attr))
                pass

            for attr in ['category', 'short_help']:
                self.assertTrue(hasattr(cmd, attr),
                                '%s command should have a %s attribute' %
                                (name, attr))
                value = getattr(cmd, attr)
                self.assertEqual(str, type(value),
                                '%s command %s attribute should be a string' %
                                (name, attr))
                pass

            self.assertEqual(str, type(cmd.name))
            self.assertEqual(tuple, type(cmd.aliases),
                             '%s aliases should be a tuple type' %
                             repr(cmd.aliases))
            for value in cmd.aliases:
                self.assertEqual(str, type(value),
                                '%s command aliases should be strings' %
                                 name)

            if cmd.min_args is not None:
                if cmd.max_args is not None:
                    self.assertTrue(cmd.min_args <= cmd.max_args,
                                    "%s min_args: %d, max_args: %d" %
                                    (name, cmd.min_args, cmd.max_args))
                    pass
                pass
            pass

    def test_args_split(self):
        for test, expect in (
            ("Now is the time",    [['Now', 'is', 'the', 'time']]),
            ("Now is the time ;;", [['Now', 'is', 'the', 'time'], []]),
            ("Now is 'the time'",  [['Now', 'is', "'the time'"]]),
            ("Now is the time ;; for all good men",
             [['Now', 'is', 'the', 'time'], ['for', 'all', 'good', 'men']]),
            ("Now is the time ';;' for all good men",
             [['Now', 'is', 'the', 'time', "';;'",
               'for', 'all', 'good', 'men']]) ):
            self.assertEqual(expect, Mcmdproc.arg_split(test))
            pass
        return

    def test_preloop_hooks(self):
        fn = self.cp.commands['list']
        self.assertEqual(0, len(self.cp.preloop_hooks),
                         'Should start out with no preloop hooks')
        self.assertEqual(False, self.cp.remove_preloop_hook(fn),
                         'Should not be able to return a non-existent hook')
        self.cp.add_preloop_hook(fn)
        self.assertEqual(1, len(self.cp.preloop_hooks),
                         'Should now have one preloop hook added')
        self.assertEqual(True, self.cp.remove_preloop_hook(fn))
        self.assertEqual(0, len(self.cp.preloop_hooks),
                         'Should be back to no preloop hooks')
        # FIXME try adding and running a couple of hooks.
        return

if __name__ == '__main__':
    unittest.main()
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013-2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.cmdlist import parse_location


class EditCommand(DebuggerCommand):
    """**edit** *location*

Edit specified file or module.
With no argument, edits file containing most recent line listed.

See also:
---------

`list`
"""

    aliases = ("ed",)
    short_help = "Edit specified file or module"

    DebuggerCommand.setup(
        locals(), category="files", max_args=1,
    )

    def run(self, args):
        curframe = self.proc.curframe
        if len(args) == 1:
            if curframe is None:
                self.errmsg(
                    "edit: no stack to pick up position from. "
                    "Use edit FILE:LINE form."
                )
                return
            filename = curframe.f_code.co_filename
            lineno = curframe.f_lineno
        elif len(args) > 1:
            location = parse_location(self.proc, args)
            if not location:
                return
            if not location.path:
                return
            filename = location.path
            if not location.line_number:
                return
            lineno = location.line_number
        editor = "ex"
        if "EDITOR" in os.environ:
            editor = os.environ["EDITOR"]
            pass
        if os.path.exists(filename):
            os.system("%s +%d %s" % (editor, lineno, filename))
        else:
            self.errmsg("edit: file %s doesn't exist" % filename)
            pass
        return

    pass


if __name__ == "__main__":

    def doit(cmd, a):
        cmd.proc.current_command = " ".join(a)
        print(cmd.run(a))

    import sys
    from trepan.debugger import Trepan

    d = Trepan()
    cmd = EditCommand(d.core.processor)
    cmd.proc.curframe = sys._getframe()
    for c in (
        ["edit"],
        ["edit", "./edit.py:34"],
        ["edit", "./noogood.py"],
    ):
        doit(cmd, c)
        pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013-2014 Rocky Bernstein
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#    02110-1301 USA.
# 'Helper' function for Processor. Put here so we
# can use this in a couple of processors.

from tracer import EVENT2SHORT

from trepan import vprocessor as Mprocessor


class PrintProcessor(Mprocessor.Processor):
    """ A processor that just prints out events as we see them. This
    is suitable for example for line/call tracing. We assume that the
    caller is going to filter out which events it wants printed or
    whether it wants any printed at all.
    """
    def __init__(self, debugger, opts=None):
        Mprocessor.Processor.__init__(self, debugger)
        return

    def event_processor(self, frame, event, arg):
        'A simple event processor that prints out events.'
        out = self.debugger.intf[-1].output
        lineno = frame.f_lineno
        filename = self.core.canonic_filename(frame)
        filename = self.core.filename(filename)
        if not out:
            print("%s - %s:%d" % (event, filename, lineno))
        else:
            out.write("%s - %s:%d" % (event, filename, lineno))
            if arg is not None:
                out.writeline(', %s ' % repr(arg))
            else:
                out.writeline('')
                pass
            pass
        return self.event_processor

    pass



import sys

import trepan.inout.stringarray
from trepan import debugger as Mdebugger
from trepan.inout import stringarray as Mstringarray


def strarray_setup(debugger_cmds):
    ''' Common setup to create a debugger with stringio attached '''
    stringin                = Mstringarray.StringArrayInput(debugger_cmds)
    stringout               = Mstringarray.StringArrayOutput()
    d_opts                  = {'input' : stringin,
                               'output': stringout}
    d                       = Mdebugger.Trepan(d_opts)
    d.settings['basename']  = True
    d.settings['different'] = False
    d.settings['autoeval']  = False
    d.settings['highlight'] = 'plain'
    return d

import re
trepan_prompt = re.compile(r'^.. \d+.*\n\(trepan3k(:.+)?\) ')
trepan_loc    = re.compile(r'^\(.+:\d+\): ')


def filter_line_cmd(a):
    '''Return output with source lines prompt and command removed'''
    # Remove extra leading spaces.
    # For example:
    # -- 42         y = 5
    # becomes
    # -- y = 5
    a1 = [re.sub(r'^(..) \d+\s+', r'\1 ', s) for s in a
         if re.match(r'^.. \d+\s+', s)]
    # Remove debugger prompts
    # For example:
    #  (trepan3k)
    a2 = [re.sub(r'\n\(trepan3k\) .*', '', s) for s in a1]

    # Remove locations (test-next.py:41): test_next_between_fn
    a3 = [re.sub(r'\n\(.*:\d+\):.*', '', s) for s in a2]
    return a3


def get_lineno():
    """Return the caller's line number"""
    caller = sys._getframe(1)
    return caller.f_lineno


def compare_output(obj, right, d, debugger_cmds):
    got = filter_line_cmd(d.intf[-1].output.output)
    if got != right:
        for i in range(len(got)):
            if i < len(right) and got[i] != right[i]:
                print("! ", got[i])
            else:
                print("  ", got[i])
                pass
            pass
        print('-' * 10)
        for i in range(len(right)):
            if i < len(got) and got[i] != right[i]:
                print("! ", right[i])
            else:
                print("  ", right[i])
                pass
            pass
        pass
    obj.assertEqual(right, got)
    return

# Demo it
if __name__=='__main__':
    print(get_lineno())
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2008, 2009, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Communication status codes"""
PRINT         = '.'
COMMAND       = 'C'   # read a command
CONFIRM_TRUE  = 'Y'
CONFIRM_FALSE = 'N'
CONFIRM_REPLY = '?'
QUIT          = 'q'
PROMPT        = 'p'
SYNC          = 's'   # Resynchronize communication
RESTART       = 'r'



# -*- coding: utf-8 -*-
#   Copyright (C) 2013, 2015, 2020 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import threading
import ctypes

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.exception import DebuggerQuit


def ctype_async_raise(thread_obj, exception):
    found = False
    target_tid = 0
    for tid, tobj in threading._active.items():
        if tobj is thread_obj:
            found = True
            target_tid = tid
            break

    if not found:
        raise ValueError("Invalid thread object")

    ret = ctypes.pythonapi.PyThreadState_SetAsyncExc(
        target_tid, ctypes.py_object(exception)
    )
    # ref: http://docs.python.org/c-api/init.html#PyThreadState_SetAsyncExc
    if ret == 0:
        raise DebuggerQuit
    elif ret > 1:
        # Huh? Why would we notify more than one threads?
        # Because we punch a hole into C level interpreter.
        # So it is better to clean up the mess.
        ctypes.pythonapi.PyThreadState_SetAsyncExc(target_tid, 0)
        raise SystemError("PyThreadState_SetAsyncExc failed")


class QuitCommand(DebuggerCommand):
    """**quit** [**unconditionally**]

Gently terminate the debugged program.

The program being debugged is aborted via a *DebuggerQuit*
exception.

When the debugger from the outside (e.g. via a `trepan` command), the
debugged program is contained inside a try block which handles the
*DebuggerQuit* exception.  However if you called the debugger was
started in the middle of a program, there might not be such an
exception handler; the debugged program still terminates but generally
with a traceback showing that exception.

If the debugged program is threaded, we raise an exception in each of
the threads ending with our own. However this might not quit the
program.

See also:
---------

See `exit` or `kill` for more forceful termination commands.

`run` and `restart` are other ways to restart the debugged program.
"""

    aliases = ("q", "quit!")
    category = "support"
    max_args = 0
    short_help = "Terminate the program - gently"

    DebuggerCommand.setup(
        locals(), category="support", max_args=0
    )

    def nothread_quit(self, arg):
        """ quit command when there's just one thread. """

        self.debugger.core.stop()
        self.debugger.core.execution_status = "Quit command"
        raise DebuggerQuit

    def threaded_quit(self, arg):
        """ quit command when several threads are involved. """
        threading_list = threading.enumerate()
        mythread = threading.currentThread()
        for t in threading_list:
            if t != mythread:
                ctype_async_raise(t, DebuggerQuit)
                pass
            pass
        raise DebuggerQuit

    def run(self, args):
        confirmed = False
        if len(args) <= 1:
            if "!" != args[0][-1]:
                confirmed = self.confirm("Really quit", False)
            else:
                confirmed = True
            pass
        if confirmed:
            threading_list = threading.enumerate()
            if (
                len(threading_list) == 1 or self.debugger.from_ipython
            ) and threading_list[0].getName() == "MainThread":
                # We are in a main thread and either there is one thread or
                # we or are in ipython, so that's safe to quit.
                return self.nothread_quit(args)
            else:
                return self.threaded_quit(args)
            pass
        return


if __name__ == "__main__":
    from trepan.processor.command import mock

    d, cp = mock.dbg_setup()
    command = QuitCommand(cp)
    try:
        command.run(["quit"])
    except DebuggerQuit:
        print("A got 'quit' a exception. Ok, be that way - I'm going home.")
        pass

    class MyThread(threading.Thread):
        def run(self):
            command.run(["quit"])
            return

        pass

    t = MyThread()
    t.start()
    t.join()
    pass
